#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include "pauls_ugly_debug.h"
#include "not_my_problem.h"

// pinouts:
//   PD1 = amplifier enable
//   PB6 = amplifier signal  (OC1B)
//   PB7 = pushbutton        (PCINT7)

#define CPU_PRESCALE(n) (CLKPR = 0x80, CLKPR = (n))
#define CPU_16MHz       0x00
#define CPU_125kHz      0x07

#define AMP_ON()	(PORTD |= (1 << 1))
#define AMP_OFF()	(PORTD &= ~(1 << 1))

#define LED_ON()	(PORTD |= (1 << 6))
#define LED_OFF()	(PORTD &= ~(1 << 6))

volatile unsigned int filepos=0;
volatile unsigned char playing=0;

void play(void)
{
	TCCR1A = (1<<COM1B1) | (1<<WGM10); // config timer1
	TCCR1B = (1<<WGM12) | (1<<CS10);  //  fast PWM mode
	TCNT1 = 0;
	OCR1BL = 128;
	_delay_ms(10);		// input filter settling time
	print("turn on amplifier\n");
	AMP_ON();
	_delay_ms(400);		// amplifier DC bias settling time
	print("begin play\n");
	filepos = 0;
	playing = 1;
	TIMSK1 = (1<<TOIE1);	// enable overflow interrupt
	LED_ON();
	sei();
	while (playing) /*wait*/ ;  // TODO: idle mode while waiting??
	cli();
	AMP_OFF();
	LED_OFF();
	//LED_OFF();
	TCCR1B = 0;
	TCCR1A = 0;
}

void wait_pushbutton_release(void)
{
	unsigned char i;
	print("wait pushbutton release\n");
	for (i=0; i<20; i++) {
		while ((PINB & 0x80) == 0) /*wait*/ ;
		_delay_ms(5);
	}
}

void sleep(void)
{
	print("sleep\n");
	PCIFR = 0x01;
	SMCR = 0x05; // powerdown
	sei();
	sleep_cpu();
	SMCR = 0;
	_delay_ms(2);
	print("wakeup\n");
	cli();
}

int main()
{

	CPU_PRESCALE(6); // 250 kHz
	PORTB = 0x80;  // pullup for pushbutton
	PORTC = 0x00;
	PORTD = 0x08;
	PORTE = 0x00;
	PORTF = 0x00;
	DDRB = 0x7F;
	DDRC = 0xFF;
	DDRD = 0xFF;
	DDRE = 0xFF;
	DDRF = 0xFF;
	_delay_ms(1);
	CPU_PRESCALE(1); // 8 MHz
	_delay_ms(1);
	print("Play 'not my problem' begin:\n");

	PCICR = 0x01;
	PCMSK0 = 0x80; // enable PCINT7

	while (1) {
		wait_pushbutton_release();
		sleep();
		wait_pushbutton_release();
		play();
		_delay_ms(10);
	}
}


ISR(TIMER1_OVF_vect)
{
	static unsigned char count=0;

	if (++count >= 4) {
		count = 0;
		if (filepos < sizeof(file_not_my_problem)) {
			OCR1BL = pgm_read_byte(file_not_my_problem + filepos) + 128;
			filepos++;
		} else {
			TIMSK1 = 0;  // disable any more interrupts
			playing = 0;
		}
	}
}

ISR(PCINT0_vect)
{
}

